#include "etherxd.h"
#include "dlfcn.h"

/* code for dynamically loading transports */

void dso_init()
{
    xmlnode next;

    debug("dso_init");

    next = xmlnode_get_firstchild(config);
    while(next != NULL)
    {           
        if(j_strcmp(xmlnode_get_name(next),"transport") == 0 && xmlnode_get_attrib(next,"dso") != NULL && xmlnode_get_attrib(next,"namespace") != NULL)
            dso_init_load(next);
        next = xmlnode_get_nextsibling(next);
    }
}

void dso_init_load(xmlnode x)
{
    void (*etherx_receive)(epacket e);
    void (*etherx_setup)(void *send, void *receive, void *alias, char *namespace);
    void *tp_so;
    char *defso;
    epacket e;
    xmlnode reg;
    edb d;

    debug("dso_init_load: %s",xmlnode_get_attrib(x,"dso"));

    defso = xmlnode_get_attrib(x,"dso");
    tp_so = dso_open(defso);
    if(tp_so == NULL && defso[0] != '/')
    {
        /* try to check for it in sys default dir */
        defso = strgrow(NULL,MODLIBDIR,0,0);
        defso = strgrow(defso,"/",1,0);
        defso = strgrow(defso,xmlnode_get_attrib(x,"dso"),1,0);
        tp_so = dso_open(defso);
        free(defso);
    }
    if(tp_so == NULL)
    {
        error("etherxd","DSO load of %s transport failed",xmlnode_get_attrib(x,"id"));
        return;
    }

    etherx_receive = dso_sym(tp_so,"etherx_receive");
    if(etherx_receive == NULL)
    {
        error("etherxd","DSO sym of %s transport failed",xmlnode_get_attrib(x,"id"));
        return;
    }
    etherx_setup = dso_sym(tp_so,"_etherx_setup");
    if(etherx_setup == NULL)
    {
        error("etherxd","DSO sym of %s transport failed",xmlnode_get_attrib(x,"id"));
        return;
    }

    (etherx_setup)(manager_send, etherx_receive, dso_alias, xmlnode_get_attrib(x, "namespace")); /* tell libetherx stuff */
    (etherx_receive)(NULL); /* tell transport to start up */

    /* HACKERY, but it works nicely... copy the registry for this transport, break in and change the name */
    reg = xmlnode_dup(x);
    reg->name = pstrdup(reg->p, "etherx:registry");
    e = epacket_new(reg,NULL,NULL,NULL);
    (etherx_receive)(e);
    xmlnode_free(reg);
}

void *dso_open(char *so)
{
    char *dlerr;
    void *so_h;

    if(so == NULL)
        return NULL;

    so_h = dlopen(so,RTLD_LAZY);
    dlerr = dlerror();
    if(dlerr != NULL)
    {
        debug("dso_open(%s) failed: %s",so, dlerr);
        return NULL;
    }

    return so_h;
}

void *dso_sym(void *so, char *func)
{
    char *dlerr;
    void *func_h;

    if(so == NULL || func == NULL)
        return NULL;

    func_h = dlsym(so,func);
    dlerr = dlerror();
    if(dlerr != NULL)
    {
        debug("dso_sym(%s) failed: %s",func, dlerr);
        return NULL;
    }

    return func_h;
}

void dso_alias(void (*handler)(epacket e), char *host, char *namespace)
{
    edb d;

    d = db(CMD_SET, host, namespace);
    d->dso_handler = handler;
    d->type = DB_TRANSPORT;
}

