#include "etherxd.h"

/* this is the intake feed from transports to deliver epackets */
void manager_send(epacket e)
{
    int s;
    edb d;
    thread t;

    debug("manager_send()");

    d = db(CMD_GET,e->to,e->ns);
    if(d == NULL || d->type == DB_ERROR)
    { 
        if(e->type == EPACKET_ERROR)
        {
            error("etherxd","can't bounce a packet to %s from %s, dropping",e->to, e->from);
            return;
        }

        if(d != NULL && d->type == DB_ERROR)
        {
            error("etherxd","%s flagged as error",e->to);
            epacket_error(e,"delivery failure");
            manager_send(e);
            return;
        }

        /* create a new connection! */
        s = make_netsocket(5269,e->to,NETSOCKET_CLIENT);
        if(s <= 0)
        {
            error("etherxd","failed to conenct to %s",e->to);
            epacket_error(e,"connect failure");
            manager_send(e);
            return;
        }else{
            t = xmlstream_new(s, (xmlstream_onNode)net_onNode);
            xmlstream_write_header(t, xmlstream_header(e->ns, e->to, NULL));
            d = db(CMD_SET,e->to,e->ns);
            d->t = t;
            d->type = DB_CONNECTION;
        }
    }

    manager_deliver(d, e);
}

/* sends epacket to the target db entry, encoding where needed for socket delivery */
void manager_deliver(edb d, epacket e)
{
    if(e == NULL)
        return;

    if(d == NULL)
    {
        error("etherxd", "invalid attempt to deliver");
        return;
    }

    if(d->type == DB_TRANSPORT)
    {
        xmlnode_put_attrib(e->x, "etherx:to", e->to);
        xmlnode_put_attrib(e->x, "etherx:from", e->from);
        if(e->type == EPACKET_ERROR)
            xmlnode_put_attrib(e->x, "etherx:error", e->error);
    }else{
        xmlnode_hide_attrib(e->x, "etherx:to");
        xmlnode_hide_attrib(e->x, "etherx:from");
    }

    if(d->dso_handler != NULL)
    {
        debug("manager_send(DSO, %s)",d->host);
        (d->dso_handler)(e);
    }else{
        debug("manager_send(SOCKET, %s)",d->host);
        xmlstream_write(d->t, e->x);
    }
}
