#include "etherxd.h"

void net_init()
{
    xmlnode self, next;
    /* look at registry and start tlisteners on appropriate IPs */
    /* create entries for own IP's in connection arrays and initi arrays */
    /* return to main thread */

    debug("net_init()");    

    self = xmlnode_get_tag(config,"etherx");	    
    if(self != NULL)
    {           
        next = xmlnode_get_firstchild(self);
        while(next != NULL)
        {
            if(j_strcmp(xmlnode_get_name(next),"listen") == 0)
            {
                net_init_listen(next);
            }
            next = xmlnode_get_nextsibling(next);
        }
    }
}

void net_init_listen(xmlnode x)
{
    char *ip;
    xmlnode x2;

    debug("net_init_listen()");
    x2 = xmlnode_get_firstchild(x);
    ip = xmlnode_get_data(x2);
    if(ip != NULL)
    {
        debug("net_init_listen: %s",ip);
        tlisten_new(5269,ip,net_onConnect,NULL);
    }
}

/* this is the callback on the listener thread whenever there is a new connect */
void net_onConnect(thread t, int sock, struct sockaddr_in sa)
{
    thread tnew;
    tstream ts;

    debug("connection established (fd: %d, ip: %s, port: %d)",
                sock, inet_ntoa(sa.sin_addr), ntohs(sa.sin_port));
    tnew = xmlstream_new(sock, (xmlstream_onNode)net_negotiate);
    ts = (tstream)(tnew->data);
    ts->data_aux4 = (void *)pstrdup(tnew->p, inet_ntoa(sa.sin_addr)); /* hide the from ip of the socket on the stream */
}

/* this is the callback on new xmlstreams, it checks/sets-up the stream
 * and determines if this is a normal connect or a new socket-based transport
 */
void net_negotiate(thread t, xmlnode in, int type)
{
    char *ns, *to;
    tstream ts = (tstream)(t->data);
    xmlnode x;
    edb d;

    if(type != XMLSTREAM_ROOT)
        return;
 
    debug("net_negotiate socket %d\n",ts->fd);
 
    ns = xmlnode_get_attrib(in, "xmlns");
    to = xmlnode_get_attrib(in, "to");
 
    if(type == XMLSTREAM_ERROR || ns == NULL || to == NULL)
    {
        xmlstream_write_header(t, xmlstream_header(NULL, NULL, NULL));
        if(type == XMLSTREAM_ERROR)
            xmlstream_error(t, "You didn't send valid XML.");
        else if(ns == NULL)
            xmlstream_error(t, "You didn't specify a default xmlns='' namespace.");
        else if(to == NULL)
            xmlstream_error(t, "You didn't send your to='host' attribute.");
        return;
    }
 
    if(j_strcmp(xmlnode_get_attrib(in, "xmlns:etherx"), ETHERX_NAMESPACE) == 0)
    { /* it's trying to be a transport */
        debug("negoitiated: new transport");
        x = xmlstream_header(ns, NULL, to);
        xmlnode_put_attrib(x, "xmlns:etherx",ETHERX_NAMESPACE);
        xmlstream_write_header(t, x);
        ts->read_evt_data = (void *)transport_onNode; /* it's a transport, let them manage it */
        transport_onNode(t, in, type); /* let the transport handler init */
    }else{
        /* it's connecting to a transport, find and hook together */
        debug("negoitiated: incoming connection");
        d = db(CMD_GET,to,ns);
        if(d == NULL || d->type != DB_TRANSPORT)
        {
            xmlstream_error(t, "This server doesn't understand your namespace.");
        }else{
            xmlstream_write_header(t, xmlstream_header(ns, NULL, to));
            ts->data_aux5 = (void *)d; /* so that transport_deliver_onNode knows which transport it's tied to */
            ts->read_evt_data = (void *)transport_deliver_onNode; /* automatically bridge this xmlstream to this transport */
        }
    }
}


/* this is a simple handler for net connections etherx has made to others */
void net_onNode(thread t, xmlnode in, int type)
{
    debug("net_onNode: some data in response to an xmlstream we initiated");
    if(type == XMLSTREAM_NODE)
    {
        if(j_strcmp(xmlnode_get_name(in),"stream:error") == 0)
        {
            error("etherxd","stream error on connection to remote server: %s",xmlnode_get_data(xmlnode_get_firstchild(in)));
        }else{
            warn("etherxd","receiving data on connection to remote server where we shouldn't be");
        }
    }
}

