#include "etherxd.h"

struct transport_dblist
{
    edb d;
    struct transport_dblist *next;
};

struct transport_data
{
    char *namespace;
    char *id;
    struct transport_dblist *d;
};

/* this is the callback for socket transports */
void transport_onNode(thread t, xmlnode in, int type)
{
    xmlnode x, next, reg;
    tstream ts = (tstream)(t->data);
    struct transport_data *td = (struct transport_data *)(ts->data_aux5);
    struct transport_dblist *td_d;
    epacket e;
    edb d;

    /* we need to
     * - remember the namespace for all the epackets we create
     * - authenticate transport for EPACKET_TRANSPORT
     * - create db entries for EPACKET_ALIAS
     * - send registry for EPACKET_REGISTRY
     * - hide etherx:to, etherx:from attribs
     * 
     */

    debug("transport: thread %d and type %d",t,type);

    if(type == XMLSTREAM_ROOT)
    {
        td = (struct transport_data *)pmalloc(t->p, sizeof(struct transport_data));
        ts->data_aux5 = (void *)td;
        td->namespace = pstrdup(t->p, xmlnode_get_attrib(in, "xmlns"));
        td->id = NULL;
        td->d = NULL;
        pth_cleanup_push(transport_cleanup, (void *)t);
        return;
    }

    if(type == XMLSTREAM_ERROR)
    {
        error("etherxd","socket connected transport sent illegal XML");
        return;
    }

    if(type == XMLSTREAM_CLOSE)
    {
        error("etherxd","socket connected transport closed connection");
        return;
    }

    x = xmlnode_dup(in);
    e = epacket_new(x, NULL, NULL, td->namespace);

    debug("transport-e: %d",e->type);

    switch(e->type)
    {
        case EPACKET_NORMAL:
            manager_send(e);
            return;
        case EPACKET_TRANSPORT:
            /* XXX actually check the secret against the config :) */
            td->id = pstrdup(t->p, xmlnode_get_data(xmlnode_get_firstchild(x)));
            debug("got etherx:transport id=%s",td->id);
            break;
        case EPACKET_ALIAS:
            if(td->id == NULL)
                break;
            /* register this thread for this host/namespace */
            d = db(CMD_SET, xmlnode_get_data(xmlnode_get_firstchild(x)), td->namespace);
            d->t = t;
            d->type = DB_TRANSPORT;
            /* save this db entry on our own list of them so we can later flag them if we dissappear */
            td_d = (struct transport_dblist *)pmalloc(t->p, sizeof(struct transport_dblist));
            td_d->d = d;
            td_d->next = td->d;
            td->d = td_d;
            debug("got etherx:alias=%s",d->host);
            break;
        case EPACKET_REGISTRY:
            if(td->id == NULL || j_strcmp(xmlnode_get_attrib(x,"type"),"request") != 0)
                break;
            /* they asked for their config, give it to em :) */
            debug("got etherx:registry request");

            next = xmlnode_get_firstchild(config);
            reg = NULL;
            while(next != NULL)
            {
                if(j_strcmp(xmlnode_get_name(next),"transport") == 0 && j_strcmp(xmlnode_get_attrib(next,"id"),td->id) == 0)
                    reg = xmlnode_dup(next);
                next = xmlnode_get_nextsibling(next);
            }

            debug("sending config data");
            if(reg != NULL)
            {
                /* HACKERY, but it works nicely... copy the registry for this transport, break in and change the name */
                reg->name = pstrdup(reg->p, "etherx:registry");
                xmlstream_write(t, reg);
                xmlnode_free(reg);
            }else{
                tstream_write(t, "<etherx:registry/>", 18);
            }
            break;
    }
    xmlnode_free(x);
}

void transport_cleanup(void *arg)
{
    thread t = (thread)arg;
    tstream ts = (tstream)(t->data);
    struct transport_data *td = (struct transport_data *)(ts->data_aux5);
    struct transport_dblist *next = td->d;

    debug("cleaning up transports db entries");

    while(next != NULL)
    {
        next->d->type = DB_ERROR;
        next = next->next;
    }
}

/* all this thing does is take data from an incoming xmlstream associated with a transport and feed it to it */
void transport_deliver_onNode(thread t, xmlnode in, int type)
{
    tstream ts = (tstream)(t->data);
    char *from_ip = (char *)(ts->data_aux4);
    edb d = (edb)(ts->data_aux5);

    debug("connection->transport");

    if(d->type == DB_ERROR)
    {
        warn("etherxd","recipient transport disconnected, dropping connection");
        xmlstream_error(t, "recipient closed connection");
    }
    if(type == XMLSTREAM_NODE)
        manager_deliver(d, epacket_new(in, d->host, from_ip, NULL));
}
