#include <jabber/jlib.h>
#include "etherx.h"

thread _etherx_thread = NULL;
void (*_etherx_receive_handler)(epacket e);
char *_etherx_namespace = NULL;

/* only called by socket connections, starts xmlstream to etherxd, returns success */
int etherx_init(char *id, char *secret, char *host, char *namespace, void (*receive)(epacket e))
{
    int s;
    xmlnode x;
    epacket e;

    /* so that etherx_init can be rerun internally and it will remember the orig args from the app */
    static char *st_id = NULL;
    static char *st_secret = NULL;
    static char *st_host = NULL;

    if(st_id == NULL)
        st_id = j_strdup(id);
    if(st_secret == NULL)
        st_secret = j_strdup(secret);
    if(st_host == NULL)
        st_host = j_strdup(host);
    if(st_host == NULL)
        st_host = j_strdup("localhost");

    if(namespace != NULL)
        _etherx_namespace = j_strdup(namespace);

    if(receive != NULL)
        _etherx_receive_handler = receive;

    s = make_netsocket(5269,st_host,NETSOCKET_CLIENT);
    if(s <= 0)
    {
        error("libetherx","etherx_connect_failure: %s",st_host);
        return -1;
    }

    _etherx_thread = xmlstream_new(s, (xmlstream_onNode)_etherx_receive);
    debug("_etherx_thread[%d] (new)\n",_etherx_thread);
    x = xmlstream_header(_etherx_namespace, st_host, NULL);
    xmlnode_put_attrib(x, "xmlns:etherx", ETHERX_NAMESPACE);
    xmlstream_write_header(_etherx_thread, x);

    x = xmlnode_new_tag("etherx:transport");
    xmlnode_put_attrib(x, "secret", st_secret);
    xmlnode_insert_cdata(x, st_id, strlen(st_id));
    xmlstream_write(_etherx_thread,x);

    x = xmlnode_new_tag("etherx:registry");
    xmlnode_put_attrib(x, "type", "request");
    xmlstream_write(_etherx_thread,x);

    if(receive != NULL)
        (*receive)(NULL); /* tell the transport to start up the first time */

    return 1;
}

void _etherx_receive(thread t, xmlnode x, int flag)
{
    epacket e;

    switch(flag)
    {
        case XMLSTREAM_ROOT:
            debug("got root tag from etherx!");
            /* ignore the root tag, transports don't use it */
            break;
        case XMLSTREAM_NODE:
            debug("got node from etherx!");
            e = epacket_new(x, NULL, NULL, NULL);
            (*_etherx_receive_handler)(e);
            break;
        case XMLSTREAM_INIT:
            pth_cleanup_push(_etherx_cleanup, (void *)t);
    }
}

void _etherx_cleanup(void *arg)
{
    /* XXX the connection died, reconnect */
    warn("libetherx","Connection to etherxd failed, attempting to reconnect...");
    sleep(2); /* give etherxd a chance to restart if it was killed */
    if(etherx_init(NULL, NULL, NULL, NULL, NULL) < 0)
    {
        error("libetherx","unable to reconnect, forcing exit!");
        exit(-1);
    }
}
